// device.cc
//    2007.6 Y.Senta

#include <stdio.h>
#include <string.h>
#include "device.h"
#include "configstr.h"
#include "common.h"

device::device(const char *c){
  m_pNext=NULL;
  m_pName=setstring(c);
  
  // default PIC16F84A
  m_nIDword=0x560;
  m_nIDAddr=0x2000;
  m_ePowerUp=VDD_BEFORE_VPP;
  m_nPMMask=0x3fff;
  m_nPMSize=0x400;
  m_nEEMemSize=0x40;
  m_nConfAddr1=0x2007;
  m_nConfAddr2=0;
  m_nConfMask1=0x3fff;
  m_nConfMask2=0x0000;
  m_nCalibAddr1=0;
  m_nCalibAddr2=0;
  m_nCalibMask1=0;
  m_nCalibMask2=0;
  m_eProgType=INTERNAL08;
  m_eProgTypeC=INTERNAL08;
  m_nProgWait=40;
  m_nProgWaitD=40;
  m_nProgWaitC=40;
  m_nProgLatch=1;
  m_eEraseType=BULKERASE_F84;
  m_nEraseWait=40;
  m_nPaMask=0x3ff0;
  m_bLVP=false;
}

device::~device(){
  if (m_pName)
	delete [] m_pName;
  
  if (m_pNext)
	delete m_pNext;
}

bool device::Load(FILE *fp){
  char buf[1024];
  int i,l;
  
  while (fgets(buf, sizeof(buf), fp)){
	i=0;
	
	while(buf[i]==' ' || buf[i]=='\t')
	  i++;
	
	while(buf[i]=='\n' || buf[i]=='\r')
	  continue;
	
	if (buf[i]==COMMENTCHAR)
	  continue;
	
	dbprintf(20, "Loading... %s", buf+i);
	
	if (!strncmp(buf+i, DEFENDSTR, strlen(DEFENDSTR)))
	  break;
	
	if (!strncmp(buf+i, IDWORDSTR, l=strlen(IDWORDSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nIDword)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, IDWORDSTR, l=strlen(IDWORDSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nIDword)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, IDADDRSTR, l=strlen(IDADDRSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nIDAddr)!=1)
		return false;
	  if (m_nIDAddr != 0x2000){
		fprintf(stderr, "[ERROR]: sorry but this version does not support as UserIDAddress != 0x2000\n");
		return false;
	  }
	  continue;
	}
	
	if (!strncmp(buf+i, PWUPTYPESTR, l=strlen(PWUPTYPESTR))){
	  char *p=chopstr(buf+i+l);
	  if (!strcmp(p, VDDVPPSTR))
		m_ePowerUp=VDD_BEFORE_VPP;
	  else if (!strcmp(p, VPPVDDSTR))
		m_ePowerUp=VPP_BEFORE_VDD;
	  else
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, PMMASKSTR, l=strlen(PMMASKSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nPMMask)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, PMSIZESTR, l=strlen(PMSIZESTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nPMSize)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, EESIZESTR, l=strlen(EESIZESTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nEEMemSize)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, CFGADDR1STR, l=strlen(CFGADDR1STR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nConfAddr1)!=1)
		return false;
	  if (m_nConfAddr1 != 0x2007){
		fprintf(stderr, "[ERROR]: sorry but this version does not support as ConfigAddress1 != 0x2007\n");
		return false;
	  }
	  continue;
	}
	
	if (!strncmp(buf+i, CFGADDR2STR, l=strlen(CFGADDR2STR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nConfAddr2)!=1)
		return false;
	  if (!(m_nConfAddr2 == 0 || m_nConfAddr2==0x2008) ){
		fprintf(stderr, "[ERROR]: sorry but this version does not support as ConfigAddress2 != 0 or 0x2008\n");
		return false;
	  }
	  continue;
	}
	
	if (!strncmp(buf+i, CFGMSK1STR, l=strlen(CFGMSK1STR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nConfMask1)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, CFGMSK2STR, l=strlen(CFGMSK2STR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nConfMask2)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, CALBADDR1STR, l=strlen(CALBADDR1STR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nCalibAddr1)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, CALBADDR2STR, l=strlen(CALBADDR2STR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nCalibAddr2)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, CALBMASK1STR, l=strlen(CALBMASK1STR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nCalibMask1)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, CALBMASK2STR, l=strlen(CALBMASK2STR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nCalibMask2)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, PRGTYPESTR, l=strlen(PRGTYPESTR))){
	  char *p=chopstr(buf+i+l);
	  if (!strcmp(p, INT08STR))
		m_eProgType=INTERNAL08;
	  else if (!strcmp(p, INT18STR))
		m_eProgType=INTERNAL18;
	  else if (!strcmp(p, EXT08STR))
		m_eProgType=EXTERNAL08;
	  else if (!strcmp(p, EXT18STR))
		m_eProgType=EXTERNAL18;
	  else
		return false;
	  continue;
	}
	if (!strncmp(buf+i, PRGTYPECSTR, l=strlen(PRGTYPECSTR))){
	  char *p=chopstr(buf+i+l);
	  if (!strcmp(p, INT08STR))
		m_eProgTypeC=INTERNAL08;
	  else if (!strcmp(p, INT18STR))
		m_eProgTypeC=INTERNAL18;
	  else if (!strcmp(p, EXT08STR))
		m_eProgTypeC=EXTERNAL08;
	  else if (!strcmp(p, EXT18STR))
		m_eProgTypeC=EXTERNAL18;
	  else
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, PRGWAITSTR, l=strlen(PRGWAITSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%d", &m_nProgWait)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, PRGWAITDSTR, l=strlen(PRGWAITDSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%d", &m_nProgWaitD)!=1)
		return false;
	  continue;
	}
	if (!strncmp(buf+i, PRGWAITCSTR, l=strlen(PRGWAITCSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%d", &m_nProgWaitC)!=1)
		return false;
	  continue;
	}
	if (!strncmp(buf+i, PRGLATCHSTR, l=strlen(PRGLATCHSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%d", &m_nProgLatch)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, ERSTYPESTR, l=strlen(ERSTYPESTR))){
	  char *p=chopstr(buf+i+l);
	  if (!strcmp(p, BESF509STR))
		m_eEraseType=BULKERASE_F509;
	  else if (!strcmp(p, BESF675STR))
		m_eEraseType=BULKERASE_F675;
	  else if (!strcmp(p, BESF683STR))
		m_eEraseType=BULKERASE_F683;
	  else if (!strcmp(p, BESF84STR))
		m_eEraseType=BULKERASE_F84;
	  else if (!strcmp(p, BESF648ASTR))
		m_eEraseType=BULKERASE_F648A;
	  else if (!strcmp(p, CESF88STR))
		m_eEraseType=CHIPERASE_F88;
	  else if (!strcmp(p, CESF77STR))
		m_eEraseType=CHIPERASE_F77;
	  else
		return false;
	  continue;
	}
	if (!strncmp(buf+i, ERSWAITSTR, l=strlen(ERSWAITSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%d", &m_nEraseWait)!=1)
		return false;
	  continue;
	}
	
	if (!strncmp(buf+i, PAMASKSTR, l=strlen(PAMASKSTR))){
	  char *p=chopstr(buf+i+l);
	  if (sscanf(p, "%x", &m_nPaMask)!=1)
		return false;
	  continue;
	}
	return false;
  }

  if (m_nConfAddr2 && m_nConfMask1 != m_nConfMask2){
	fprintf(stderr, "[ERROR]: sorry but this version does not support as ConfigMask1 != ConfigMask2 when ConfigAddress2 == 0x2008\n");
	return false;

  }
  
  return true;
}
