// piccommand.cc
//    2007.6 Y.Senta

#include <stdio.h>
#include <string.h>
#include "piccommand.h"
#include "picmemory.h"
#include "device.h"
#include "serialport.h"
#include "common.h"

#define TRYTIME 4
#define WRITER509 "W509"
#define WRITER648 "W648"

#define W_BASE      0x00
#define W_SINGLE    0x01
#define W_RECDAT    0x02
#define W_INTERNAL  0x03
#define W_SENDDATA  0x04
#define W_EXTERNAL  0x05
#define W_SINGLE_N  0x06
#define W_RECDAT_N  0x07
#define W_ADVANC_W  0x08
#define W_SEND20    0x09
#define W_RECV8     0x0a
#define W_ADDRSET24 0x0b
#define W_TABLE_R_N 0x0c
#define W_WDATA_7   0x0d
#define W_EP_ADDR   0x0e
#define W_EP_READ_N 0x0f
#define W_EP_WRITE  0x10
#define W_EXP_BASE  0xff

#define W2_VERSION  W_BASE  // 0x00
#define W2_VDDUP    0x01
#define W2_VDDDN    0x02
#define W2_VDDUP2   0x03
#define W2_VDDDN2   0x04
#define W2_CHGBPS   0xff

#define P_LOAD_CONFIG 0x00
#define P_LOAD_PRGMEM 0x02
#define P_LOAD_DATMEM 0x03
#define P_READ_PRGMEM 0x04
#define P_READ_DATMEM 0x05
#define P_INC_ADDR    0x06
#define P_BEGIN_PROG  0x08
#define P_ERASE_PGRMEM 0x09
#define P_ERASE_DATMEM 0x0b

#define PROG_VERIFY  0x20

bool piccommand::CheckWriter(){
  m_bExpWriter=false;
  
  char buf[1024];
  int n=0;
  
  for (int i=0;i<TRYTIME;i++){
	if (!m_pSerial->SendCmd(W_BASE)){
	  fprintf(stderr, "[ERROR] May not be Writer509\n");
	  return false;
	}
	
	n=m_pSerial->RecvChar(buf, sizeof(buf));
	if (n)
	  break;
  }
  buf[n]=0;
  
  dbprintf(3, "Check Writer 00-00 [%s]\n", buf);
  
  if (strncmp(buf, WRITER509, strlen(WRITER509))){
  	fprintf(stderr, "[ERROR] May not be a Writer509\n");
	return false;
  }
  if (!m_bQuite)
	fprintf(stderr, "Check Writer509... OK (%s)", buf);
  
  if (m_pSerial->SendCmd(W_EXP_BASE) && m_pSerial->SendCmd(W2_VERSION)){
	n=m_pSerial->RecvChar(buf, sizeof(buf));
	buf[n]=0;
	
	dbprintf(3, "Check Writer FF-00 [%s]\n", buf);
	
	if (!strncmp(buf, WRITER648, strlen(WRITER648))){
	  if (!m_bQuite)
		fprintf(stderr, ", Writer648... OK (%s)", buf);
	  m_bExpWriter=true;
	}
  }
  if (!m_bQuite)
	fprintf(stderr, "\n");
  return true;
}

void CommunicateErr(){
  fprintf (stderr, "[Error]: Communicate Error\n");
}

bool piccommand::ChangeSpeed(){
  if (m_pSerial->SendCmd(W_EXP_BASE) && 
	  m_pSerial->SendCmd(W2_CHGBPS) &&
	  m_pSerial->SendCmd(m_nNewBps)){
	return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::BackupCalib(){
  PowerON();
  int tmp;
  
  if (m_pDevice->GetCalibAddr1()>0x2000){
	LoadConfig(0);
	Increment(m_pDevice->GetCalibAddr1()-0x2000);
	if (!ReadProgMemory(&tmp)){
	  PowerDown();
	  fprintf(stderr, " [ERROR] BackupCalib1\n");
	  return false;
	}
	m_Mem.m_nCalibData1 = tmp & m_pDevice->GetCalibMask1();
  } else if (m_pDevice->GetCalibAddr1()>0x2000 > 0){
	if (m_pDevice->GetConfAddr1() < 0x1000)
	  Increment();
	Increment(m_pDevice->GetCalibAddr1());
	if (!ReadProgMemory(&tmp)){
	  PowerDown();
	  fprintf(stderr, " [ERROR] BackupCalib2\n");
	  return false;
	}
	m_Mem.m_nCalibData1 = tmp & m_pDevice->GetCalibMask1();
  }
  
  if (m_pDevice->GetCalibAddr2()>0x2000){
	LoadConfig(0);
	Increment(m_pDevice->GetCalibAddr2()-0x2000);
	if (!ReadProgMemory(&tmp)){
	  PowerDown();
	  fprintf(stderr, " [ERROR] BackupCalib3\n");
	  return false;
	}
	m_Mem.m_nCalibData1 = tmp & m_pDevice->GetCalibMask2();
  } else if (m_pDevice->GetCalibAddr2()>0x2000 > 0){
	if (m_pDevice->GetConfAddr1() < 0x1000)
	  Increment();
	Increment(m_pDevice->GetCalibAddr2());
	if (!ReadProgMemory(&tmp)){
	  PowerDown();
	  fprintf(stderr, " [ERROR] BackupCalib4\n");
	  return false;
	}
	m_Mem.m_nCalibData2 = tmp & m_pDevice->GetCalibMask2();
  }
  
  PowerDown();
  return true;
}

bool piccommand::RestoreCalib(){
  PowerON();
  
  if (m_pDevice->GetCalibAddr1()>0x2000){
	LoadConfig(0);
	Increment(m_pDevice->GetCalibAddr1()-0x2000);
	if (!LoadProgMemory(m_Mem.m_nCalibData1) && !SendBeginProg(WAIT_PROG)){
	  PowerDown();
	  fprintf(stderr, " [ERROR] RestoreCalib1\n");
	  return false;
	}
  } else if (m_pDevice->GetCalibAddr1()>0x2000 > 0){
	if (m_pDevice->GetConfAddr1() < 0x1000)
	  Increment();
	Increment(m_pDevice->GetCalibAddr1());
	if (!LoadProgMemory(m_Mem.m_nCalibData1) && !SendBeginProg(WAIT_PROG)){
	  PowerDown();
	  fprintf(stderr, " [ERROR] RestoreCalib2\n");
	  return false;
	}
  }
  
  if (m_pDevice->GetCalibAddr2()>0x2000){
	LoadConfig(0);
	Increment(m_pDevice->GetCalibAddr2()-0x2000);
	if (!LoadProgMemory(m_Mem.m_nCalibData2) && !SendBeginProg(WAIT_PROG)){
	  PowerDown();
	  fprintf(stderr, " [ERROR] RestoreCalib3\n");
	  return false;
	}
  } else if (m_pDevice->GetCalibAddr2()>0x2000 > 0){
	if (m_pDevice->GetConfAddr1() < 0x1000)
	  Increment();
	Increment(m_pDevice->GetCalibAddr2());
	if (!LoadProgMemory(m_Mem.m_nCalibData2) && !SendBeginProg(WAIT_PROG)){
	  PowerDown();
	  fprintf(stderr, " [ERROR] BackupCalib4\n");
	  return false;
	}
  }
  
  PowerDown();
  return true;
}

bool piccommand::InternalTime(int c, int t){
  if (m_pSerial->SendCmd(W_INTERNAL) && 
	  m_pSerial->SendCmd(c) &&
	  m_pSerial->SendCmd(t)){
	return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::ExternalTime(int c1, int c2, int t){
  if (m_pSerial->SendCmd(W_EXTERNAL) && 
	  m_pSerial->SendCmd(c1) &&
	  m_pSerial->SendCmd(c2) &&
	  m_pSerial->SendCmd(t)){
	return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::Increment(){
  if (m_pSerial->SendCmd(W_SINGLE) && 
	  m_pSerial->SendCmd(P_INC_ADDR))
	return true;
  CommunicateErr();
  return false;
}

bool piccommand::Increment(int n){
  if (m_pSerial->SendCmd(W_SINGLE_N) && 
	  m_pSerial->SendCmd(P_INC_ADDR) &&
	  m_pSerial->SendCmd(n & 0xff) &&
	  m_pSerial->SendCmd((n >> 8) & 0xff)){
	return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::Setup(int c){
  if (m_pSerial->SendCmd(W_SINGLE) && 
	  m_pSerial->SendCmd(c)){
	return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::LoadConfig(int n){
  if (m_pSerial->SendCmd(W_SENDDATA) && 
	  m_pSerial->SendCmd(P_LOAD_CONFIG) &&
	  m_pSerial->SendCmd(n & 0xff) &&
	  m_pSerial->SendCmd((n >> 8) & 0xff)){
	return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::ReadProgMemory(int *r){
  char c1, c2;
  
  if (m_pSerial->SendCmd(W_RECDAT) && 
	  m_pSerial->SendCmd(P_READ_PRGMEM) &&
	  m_pSerial->RecvChar(&c1) && 
	  m_pSerial->RecvChar(&c2)){
	
	*r=(c1 & 0xff) + c2*0x100;
	
	return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::LoadProgMemory(int d){
  if (m_pSerial->SendCmd(W_SENDDATA) && 
	  m_pSerial->SendCmd(P_LOAD_PRGMEM) &&
	  m_pSerial->SendCmd(d & 0xff) && 
	  m_pSerial->SendCmd((d >> 8) & 0xff)){
	
	return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::LoadProgMemory8(int *p){
  if (m_pSerial->SendCmd(W_ADVANC_W)){
	int sum;
	char c1,c2,c3, cx;
	switch(m_pDevice->GetProgType()){
	case INTERNAL08:
	  c1=P_LOAD_PRGMEM;
	  c2=P_BEGIN_PROG;
	  c3=0x00;
	  break;
	case INTERNAL18:
	  c1=P_LOAD_PRGMEM;
	  c2=0x18;
	  c3=0x00;
	  break;
	case EXTERNAL08:
	  c1=P_LOAD_PRGMEM;
	  c2=P_BEGIN_PROG;
	  c3=0x0e;
	  break;
	case EXTERNAL18:
	  c1=P_LOAD_PRGMEM;
	  c2=0x18;
	  c3=0x17;
	  break;
	default:
	  return false;
	}

	sum=c1+c2+c3;
	m_pSerial->SendChar(c1);
	m_pSerial->SendChar(c2);
	m_pSerial->SendChar(c3);
	sum += m_pDevice->GetProgWait();
	m_pSerial->SendChar(m_pDevice->GetProgWait());
	sum += m_pDevice->GetProgLatch()-1;
	m_pSerial->SendChar(m_pDevice->GetProgLatch()-1);

	for (int i=0;i<4;i++){
	  sum += p[i];
	  m_pSerial->SendChar(p[i] & 0xff);
	  sum += p[i] >> 8;
	  m_pSerial->SendChar((p[i]>>8) & 0xff);
	}
	m_pSerial->RecvChar(&cx);

	dbprintf(15, "ADV Chksum1: %02x %02x\n", cx & 0xff, sum & 0xff);

	sum -= cx;
	if (sum & 0xff)
	  fprintf(stderr, "[ERROR] advanced checksum error1\n");

	if (m_pSerial->SendCmd(W_ADVANC_W)){
	  sum=c1+c2+c3;
	  m_pSerial->SendChar(c1);
	  m_pSerial->SendChar(c2);
	  m_pSerial->SendChar(c3);
	  sum += m_pDevice->GetProgWait();
	  m_pSerial->SendChar(m_pDevice->GetProgWait());
	  if (m_pDevice->GetProgLatch()==8){
		sum += 3;
		m_pSerial->SendChar(3);
	  } else {
		sum += m_pDevice->GetProgLatch()-1;
		m_pSerial->SendChar(m_pDevice->GetProgLatch()-1);
	  }
	  for (int i=4;i<8;i++){
		sum += p[i];
		m_pSerial->SendChar(p[i] & 0xff);
		sum += p[i] >> 8;
		m_pSerial->SendChar((p[i]>>8) & 0xff);
	  }
	  m_pSerial->RecvChar(&cx);

	  dbprintf(15, "ADV Chksum: %02x %02x\n", cx & 0xff, sum & 0xff);

	  sum -= cx;
	  if (sum & 0xff)
		fprintf(stderr, "[ERROR] advanced checksum error2\n");
	
	  return true;
	}
  }
  return false;
}

bool piccommand::ReadDataMemory(int *r){
  char c1, c2;
  
  if (m_pSerial->SendCmd(W_RECDAT) && 
	  m_pSerial->SendCmd(P_READ_DATMEM) &&
	  m_pSerial->RecvChar(&c1) && 
	  m_pSerial->RecvChar(&c2)){
	
	*r= (c1 & 0xff) + c2*0x100;
	
	return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::PowerON(){
  if (m_bLVP){
	if (!m_pDevice->isSupportLVP()){
	  fprintf (stderr, "[ERROR]: this device not support LVP\n");
	  return false;
	}
	if (!m_bExpWriter){
	  fprintf (stderr, "[ERROR]: this writer not support LVP\n");
	  return false;
	}
	if (m_pSerial->SendCmd(W_EXP_BASE) && 
		m_pSerial->SendCmd(W2_VDDUP))
	  return true;
	CommunicateErr();
	return false;
  }
  
  if (m_pDevice->GetPowerupType() == VDD_BEFORE_VPP){
	if (m_pSerial->SendCmd(W_BASE) && m_pSerial->SendCmd(W2_VDDUP))
	  return true;
  } else {
	if (m_pSerial->SendCmd(W_BASE) && m_pSerial->SendCmd(W2_VDDUP2))
	  return true;
  }
  CommunicateErr();
  return false;
}

bool piccommand::PowerDown(){
  if (m_bLVP){
	if (m_pSerial->SendCmd(W_EXP_BASE) && 
		m_pSerial->SendCmd(W2_VDDDN))
	  return true;
	
	return PowerDownTryAgain();
  }
  
  if (m_pDevice->GetPowerupType() == VDD_BEFORE_VPP){
	if (m_pSerial->SendCmd(W_BASE) && m_pSerial->SendCmd(W2_VDDDN))
	  return true;
  } else {
	if (m_pSerial->SendCmd(W_BASE) && m_pSerial->SendCmd(W2_VDDDN2))
	  return true;
  }
  return PowerDownTryAgain();
}

bool piccommand::PowerDownTryAgain(){
  char buf[1024], tmp;
  int n=0;

  while (m_pSerial->RecvChar(&tmp));

  for (int i=0;i<TRYTIME;i++){
	m_pSerial->SendCmd(W_BASE);
	n=m_pSerial->RecvChar(buf, sizeof(buf));
	if (n)
	  break;
  }
  buf[n]=0;

  if (strncmp(buf, WRITER509, strlen(WRITER509)))
	return false;

  if (m_bLVP)
	m_pSerial->SendCmd(W_EXP_BASE);
  else
	m_pSerial->SendCmd(W_BASE);

  if (!m_pSerial->SendCmd(W2_VDDDN))
	return false;

  return true;
}

bool piccommand::BulkeraseF509(){
  return InternalTime(P_ERASE_PGRMEM, m_pDevice->GetEraseWait());
}

bool piccommand::BulkeraseF675(){
  if (LoadConfig(0) &&
	  InternalTime(P_ERASE_PGRMEM, m_pDevice->GetEraseWait()) && 
	  InternalTime(P_ERASE_DATMEM, m_pDevice->GetEraseWait()))
	return true;
  return false;
}

bool piccommand::BulkeraseF683(){
  if (InternalTime(P_ERASE_PGRMEM, m_pDevice->GetEraseWait()) &&
	  LoadConfig(0) &&
	  InternalTime(P_ERASE_PGRMEM, m_pDevice->GetEraseWait()) && 
	  InternalTime(P_ERASE_DATMEM, m_pDevice->GetEraseWait()))
	return true;
  return false;
}

bool piccommand::BulkeraseF84(){
  if (LoadConfig(0x3fff) &&
	  Increment(0x07) &&
	  Setup(0x01) &&
	  Setup(0x07) &&
	  InternalTime(P_BEGIN_PROG, m_pDevice->GetEraseWait()) &&
	  Setup(0x01) &&
	  Setup(0x07) &&
	  Setup(P_ERASE_DATMEM) &&
	  InternalTime(0x18, m_pDevice->GetEraseWait()))
	return true;
  return false;
}

bool piccommand::BulkeraseF648A(){
  if (LoadConfig(0x3fff) &&
	  InternalTime(P_ERASE_PGRMEM, m_pDevice->GetEraseWait()) && 
	  InternalTime(P_ERASE_DATMEM, m_pDevice->GetEraseWait()))
	return true;
  return false;
}

bool piccommand::ChiperaseF88(){
  if (LoadConfig(0x3fff) &&
	  InternalTime(0x1f, m_pDevice->GetEraseWait()))
	return true;
  return false;
}

bool piccommand::ChiperaseF77(){
  if (LoadConfig(0x3fff) &&
	  InternalTime(P_ERASE_PGRMEM, m_pDevice->GetEraseWait()))
	return true;
  return false;
}

bool piccommand::DeviceCheck(){
  if (!m_pDevice->GetDeviceID())
	return true;
  
  PowerON();
  if (!LoadConfig(0) || !Increment(6)){
	fprintf(stderr, "[ERROR] device check\n");
	PowerDown();
	return false;
  }
  
  int ret;
  ReadProgMemory(&ret);
  PowerDown();
  
  dbprintf(1, "DeviceCheck %04x -- %04x\n", m_pDevice->GetDeviceID(), ret);
  
  if (m_pDevice->GetDeviceID() != (ret & 0x3fe0)){
	fprintf(stderr, "[ERROR]: device not match\n");
	return false;
  }
  
  return true;
}

bool piccommand::ChipErase(){
  if (!m_bQuite)
	fprintf(stderr, "Chip Erase...");
  
  if (m_pDevice->GetCalibAddr1()){
	if (!BackupCalib())
	  return false;
  }
  
  PowerON();
  
  bool ret=false;
  switch(m_pDevice->GetEraseType()){
  case BULKERASE_F509:
	ret=BulkeraseF509();
	break;
  case BULKERASE_F675:
	ret=BulkeraseF675();
	break;
  case BULKERASE_F683:
	ret=BulkeraseF683();
	break;
  case BULKERASE_F84:
	ret=BulkeraseF84();
	break;
  case BULKERASE_F648A:
	ret=BulkeraseF648A();
	break;
  case CHIPERASE_F88:
	ret=ChiperaseF88();
	break;
  case CHIPERASE_F77:
	ret=ChiperaseF77();
	break;
  default:
	fprintf(stderr, " unknown erase type\n");
	break;
  }
  
  if (ret && m_pDevice->GetCalibAddr1())
	ret=RestoreCalib();
  
  PowerDown();
  
  if (!m_bQuite)
	fprintf(stderr, "Done\n");
  
  return ret;
}

bool piccommand::ReadBuffer(){
  PowerON();
  
  int nSize, nMask, *pBuffer;
  char h,l;
  
  if (!m_bQuite)
	fprintf(stderr, "Program Area Reading");
  
  nSize=m_pDevice->GetProgSize();
  nMask=m_pDevice->GetPMMask();
  pBuffer=m_Mem.GetProgMem();
  if (!(m_pSerial->SendCmd(W_RECDAT_N) && 
		m_pSerial->SendCmd(P_READ_PRGMEM) && 
		m_pSerial->SendCmd(nSize & 0xff) && 
		m_pSerial->SendCmd((nSize >> 8) & 0xff))){
	PowerDown();
	return false;
  }
  
  for (int i=0;i<nSize;i++){
	if (m_pSerial->RecvChar(&l) && m_pSerial->RecvChar(&h)) {
	  pBuffer[i]=(h*0x100 + (l & 0xff)) & nMask;
	} else {
	  fprintf(stderr, "Error\n");
	  PowerDown();
	  return false;
	}
	if (!m_bQuite && i%0x20 == 0)
	  fprintf(stderr, ".");
  }
  
  if (!m_bQuite)
	fprintf(stderr, "Done\n");
  
  if (m_pSerial->RecvChar(&l)){
	fprintf(stderr, "communication error [%02x]\n", l);
	PowerDown();
	return false;
  }
  
  int nData;
  if (m_pDevice->GetDataSize()){
	if (!m_bQuite)
	  fprintf(stderr, "Data Area Reading");
	
	nSize=m_pDevice->GetDataSize();
	nMask=DATAMASK;
	pBuffer=m_Mem.GetDataMem();
	
	for (int i=0;i<nSize;i++){
	  if (!ReadDataMemory(&nData)){
		PowerDown();
		return false;
	  }
	  pBuffer[i]=nData & nMask;
	  Increment();
	  
	  if (!m_bQuite && i%0x10 == 0)
		fprintf(stderr, ".");
	}
	
	if (!m_bQuite)
	  fprintf(stderr, "Done\n");
  }
  
  nSize=m_pDevice->GetConfSize();
  nMask=m_pDevice->GetConfMask1();
  pBuffer=m_Mem.GetConfMem();
  
  if (!m_bQuite)
	fprintf(stderr, "Config Area Reading");
  
  if (!LoadConfig(0)){
	PowerDown();
	return false;
  }
  
  for (int i=0;i<nSize;i++){
	if (!ReadProgMemory(&nData)){
	  PowerDown();
	  return false;
	}
	pBuffer[i]=nData & nMask;
	Increment();
	if (!m_bQuite)
	  fprintf(stderr, ".");
  }
  
  if (!m_bQuite)
	fprintf(stderr, "Done\n");
  
  PowerDown();
  return true;
}

bool piccommand::SendBeginProg(WAITTYPE eWait){
  int nWait;
  switch (eWait){
  case WAIT_PROG:
	nWait=m_pDevice->GetProgWait();
	break;
  case WAIT_DATA:
	nWait=m_pDevice->GetProgWaitD();
	break;
  case WAIT_CONF:
	nWait=m_pDevice->GetProgWaitC();
	break;
  default:
	fprintf(stderr, "[*] Param Error piccommand::SendBeginProg()\n");
	return false;
  }

  bool bRet=false;
  switch(m_pDevice->GetProgType()){
  case INTERNAL08:
	bRet=InternalTime(P_BEGIN_PROG, nWait);
	break;
  case INTERNAL18:
	bRet=InternalTime(0x18, nWait);
	break;
  case EXTERNAL08:
	bRet=ExternalTime(P_BEGIN_PROG, 0x0e, nWait);
	break;
  case EXTERNAL18:
	bRet=ExternalTime(0x18, 0x17, nWait);
	break;
  default:
	break;
  }
  if (!bRet)
	fprintf (stderr, "[ERROR]: Program2\n");
  
  return bRet;
}

bool piccommand::WriteBuffer(){
  PowerON();
  
  int nSize, nMask, *pBuffer;
  
  if (!m_bQuite)
	fprintf(stderr, "Program Area Writing");
  
  nSize=m_pDevice->GetProgSize();
  nMask=m_pDevice->GetPMMask();
  pBuffer=m_Mem.GetProgMem();
 
  if (m_pDevice->GetCalibAddr1() && m_pDevice->GetCalibAddr1()<0x1000)
	nSize--;
 
  bool bAdv15=m_bAdvance;
  int nIncrement=0;

  for (int i=0;i<nSize;){
	if (!m_bQuite && i%0x20 == 0){
	  fprintf(stderr, ".");
	}

	if (pBuffer[i] != nMask){
	  if (nIncrement){
		if (nIncrement==1)
		  Increment();
		else
		  Increment(nIncrement);
		nIncrement=0;
	  }

	  // I don't know why condition i%8 is necessary but it needs.
	  if (bAdv15 && i%8==0 && i < nSize-8){
		if ( LoadProgMemory8(pBuffer+i)){
		  i+=8;
		  continue;
		}
		bAdv15=false;
	  }

	  if (LoadProgMemory(pBuffer[i])){
		if (!SendBeginProg(WAIT_PROG)){
		  PowerDown();
		  return false;
		}
	  } else {
		PowerDown();
		return false;
	  }
	}
	i++;
	nIncrement++;
  }

  if (!m_bQuite)
	fprintf(stderr, "Done\n");
  
  if (m_pDevice->GetDataSize()){
	if (!m_bQuite)
	  fprintf(stderr, "Data Area Writing");
	
	nSize=m_pDevice->GetDataSize();
	nMask=DATAMASK;
	pBuffer=m_Mem.GetDataMem();
	
	nIncrement=0;

	for (int i=0;i<nSize;i++){
	  if (pBuffer[i] != 0xff){
		if (nIncrement){
		  if (nIncrement==1)
			Increment();
		  else
			Increment(nIncrement);
		  nIncrement=0;
		}

		if (!(m_pSerial->SendCmd(W_SENDDATA) && 
			  m_pSerial->SendCmd(P_LOAD_DATMEM) &&
			  m_pSerial->SendCmd(pBuffer[i] & 0xff) &&
			  m_pSerial->SendCmd((pBuffer[i] >> 8) & 0xff))){
		  fprintf (stderr, "[ERROR]: Program3\n");
		  PowerDown();
		  return false;
		}
		if (!SendBeginProg(WAIT_DATA)){
		  PowerDown();
		  return false;
		}
	  }
	  nIncrement++;
	  if (!m_bQuite && i%0x10 == 0)
		fprintf(stderr, ".");
	}
  }
  
  if (!m_bQuite)
	fprintf(stderr, "Done\n");
  
  nSize=m_pDevice->GetConfSize();
  nMask=m_pDevice->GetConfMask1();
  pBuffer=m_Mem.GetConfMem();
  
  if (!m_bQuite)
	fprintf(stderr, "Config Area Writing");
  
  LoadConfig(pBuffer[0]);
  for (int i=0;i<4;i++){
	if (!LoadProgMemory((pBuffer[i] & 0x0f))){
	  fprintf (stderr, "[ERROR]: Program1\n");
	  PowerDown();
	  return false;
	}
	if (!SendBeginProg(WAIT_PROG)){
	  PowerDown();
	  return false;
	}
	Increment();
  }
  if (!m_bQuite)
	fprintf(stderr, ".");

  Increment(3);
  
  int nConf;
  if (!ReadProgMemory(&nConf)){
	PowerDown();
	return false;
  }
  
  nConf=(pBuffer[7] & m_pDevice->GetConfMask1()) | 
	(nConf & ~m_pDevice->GetConfMask1());
  if (!LoadProgMemory(nConf)){
	fprintf (stderr, "[ERROR]: Program4\n");
	PowerDown();
	return false;
  }
  if (!SendBeginProg(WAIT_CONF)){
	PowerDown();
	return false;
  }

  if (m_pDevice->GetConfAddr2()){
	Increment();
	nConf=pBuffer[8] & m_pDevice->GetConfMask2();
	if (!LoadProgMemory(nConf)){
	  fprintf (stderr, "[ERROR]: Program5\n");
	  PowerDown();
	  return false;
	}
	if (!SendBeginProg(WAIT_CONF)){
	  PowerDown();
	  return false;
	}
  }

  if (!m_bQuite)
	fprintf(stderr, ".Done\n");
  
  PowerDown();
  return true;
}

bool piccommand::VerifyBuffer(){
  PowerON();
  
  int nSize, nMask, *pBuffer;
  char h,l;
  
  if (!m_bQuite)
	fprintf(stderr, "Program Area Verifing");
  
  nSize=m_pDevice->GetProgSize();
  nMask=m_pDevice->GetPMMask();
  pBuffer=m_Mem.GetProgMem();
  
  if (m_pDevice->GetCalibAddr1() && m_pDevice->GetCalibAddr1()<0x1000)
	nSize--;

  int nNow=0;
  while (nNow < nSize){
	int nSkip=PROG_VERIFY;
	
	if (nNow+nSkip > nSize)
	  nSkip = nSize-nNow;
	
	if (!(m_pSerial->SendCmd(W_RECDAT_N) && 
		  m_pSerial->SendCmd(P_READ_PRGMEM) && 
		  m_pSerial->SendCmd(nSkip & 0xff) && 
		  m_pSerial->SendCmd((nSkip >> 8) & 0xff))){
	  PowerDown();
	  return false;
	}
	
	bool bError=false;
	for (int i=0;i<nSkip;i++){
	  if (m_pSerial->RecvChar(&l) && m_pSerial->RecvChar(&h)) {
		if (pBuffer[nNow+i] != (int)((h*0x100 + (l & 0xff)) & nMask)){
		  if (!bError){
			fprintf(stderr, "\nVerify Error on Program Area of %04x: %04x - %04x\n",
					(nNow+i)*2, pBuffer[nNow+i], (h*0x100 + (l & 0xff)) & nMask);
			bError=true;
		  }
		}
	  } else {
		fprintf(stderr, "Error\n");
		PowerDown();
		return false;
	  }
	}
	if (bError){
	  PowerDown();
	  return true;
	}
	if (!m_bQuite)
	  fprintf(stderr, ".");
	
	nNow += nSkip;
  }
  
  if (!m_bQuite)
	fprintf(stderr, "OK\n");
  
  int nData;
  if (m_pDevice->GetDataSize()){
	if (!m_bQuite)
	  fprintf(stderr, "Data Area Verifing");
	
	nSize=m_pDevice->GetDataSize();
	nMask=DATAMASK;
	pBuffer=m_Mem.GetDataMem();
	
	for (int i=0;i<nSize;i++){
	  if (!ReadDataMemory(&nData)){
		PowerDown();
		return false;
	  }
	  if (pBuffer[i]!= (int)(nData & nMask)){
		fprintf(stderr, "\nVerify Error on Data Area of %04x: %04x - %04x\n",
				i*2, pBuffer[i], nData & nMask);
		PowerDown();
		return true;
	  }
	  Increment();
	  
	  if (!m_bQuite && i%0x10 == 0)
		fprintf(stderr, ".");
	}
	
	if (!m_bQuite)
	  fprintf(stderr, "OK\n");
  }
  
  nSize=m_pDevice->GetConfSize();
  pBuffer=m_Mem.GetConfMem();
  
  if (!m_bQuite)
	fprintf(stderr, "Config Area Verifing");
  
  if (!LoadConfig(0)){
	PowerDown();
	return false;
  }
  
  for (int i=0;i<nSize;i++){
	if ((m_pDevice->GetConfAddr1() 
		 && i==m_pDevice->GetConfAddr1()-CONFMEMADDR/2) ||
		(m_pDevice->GetConfAddr2() 
		 && i==m_pDevice->GetConfAddr2()-CONFMEMADDR/2)){
	  if (!ReadProgMemory(&nData)){
		PowerDown();
		return false;
	  }

	  nMask=i==m_pDevice->GetConfAddr1()-CONFMEMADDR/2 ?
		m_pDevice->GetConfMask1() : m_pDevice->GetConfMask2();
	  
	  if (pBuffer[i] != (int)(nData & nMask)){
		fprintf(stderr, "\nVerify Error on Config Area of %04x: %04x - %04x\n",
				i*2, pBuffer[i], nData & nMask);
		PowerDown();
		return true;
	  }
	}
	
	Increment();
	if (!m_bQuite)
	  fprintf(stderr, ".");
  }
  
  if (!m_bQuite)
	fprintf(stderr, "OK\n");
  
  PowerDown();
  return true;
}

bool piccommand::Command(){
  if (!CheckWriter()){
	fprintf(stderr, "[ERROR]: PIC Writer may not connected\n");
	return false;
  }
  if (m_bCheck){
	if (m_pDevice){
	  if (DeviceCheck())
		fprintf(stderr, "[%s] connected\n", m_pDevice->GetName());
	  else
		fprintf(stderr, "[%s] not connected\n", m_pDevice->GetName());
	}
	return true;
  }
  
  if (m_bChgBps){
	if (!m_bExpWriter){
	  fprintf(stderr, "[ERROR]: This writer not support change bps\n");
	  return false;
	}
	return ChangeSpeed();
  }
  
  if (!DeviceCheck())
	return false;
  
  if (m_bErase && !ChipErase()){
	fprintf(stderr, "[ERROR]: Chip Erase\n");
	return false;
  }
  
  m_Mem.Init(m_pDevice);
  
  if (m_bRead){
    if (!ReadBuffer())
      return false;
    if (!m_Mem.Save(m_pFHexO))
      return false;
  }
  
  if (m_bWrite){
    if (!m_Mem.Load(m_pFHexI))
      return false;
    if (!WriteBuffer())
      return false;
  }
  
  if (m_bVerify){
    if (!m_bWrite && !m_Mem.Load(m_pFHexI))
      return false;
    if (!VerifyBuffer())
      return false;
  }
  
  return true;
}
