// picmemory.cc
//    2007.6 Y.Senta

#include <stdio.h>
#include <string.h>
#include "picmemory.h"
#include "device.h"
#include "common.h"

#define DUMPMEMSIZE 8

bool intelhex::GetData(const char *c){
  dbprintf(21, "%s\n", c);
  if (c[0]!=':')
    return false;
  
  int nDataNum;
  if (!gethexbyte(&nDataNum, c+1, 1))
    return false;
  
  int nOffsetAddr;
  if (!gethexbyte(&nOffsetAddr, c+3, 2))
    return false;
  
  int nType;
  if (!gethexbyte(&nType, c+7, 1))
    return false;
  
  int sum; // not use
  if (nType==0){
    int *pData=NULL;
	
    if (nOffsetAddr >= PROGMEMADDR && 
		nOffsetAddr+nDataNum <= PROGMEMADDR+m_pMem->GetProgSize2()){
	  pData=m_pMem->GetProgMem();
	  nOffsetAddr-=PROGMEMADDR;
	} else if (nOffsetAddr >= CONFMEMADDR &&
			   nOffsetAddr+nDataNum <= CONFMEMADDR+m_pMem->GetConfSize2()){
	  pData=m_pMem->GetConfMem();
	  nOffsetAddr-=CONFMEMADDR;
    } else if (nOffsetAddr >= DATAMEMADDR &&
			   nOffsetAddr+nDataNum <= DATAMEMADDR+m_pMem->GetDataSize2()){
	  pData=m_pMem->GetDataMem();
	  nOffsetAddr-=DATAMEMADDR;
    } else {
	  dbprintf(5, "Offset:%04x, Num:%02x, %04x %04x %04x\n",
			   nOffsetAddr, nDataNum, 
			   m_pMem->GetProgSize2(), 
			   m_pMem->GetConfSize2(),
			   m_pMem->GetDataSize2());
	  fprintf (stderr, "Memory error [%04x]\n", nOffsetAddr);
	  return false;
	}
	int h, l;
	for (int i=0;i<nDataNum;i+=2){
	  if (!gethexbyte(&l, c+9+i*2, 1))
		return false;
	  if (!gethexbyte(&h, c+11+i*2, 1))
		return false;
	  pData[(nOffsetAddr+i)/2]=h*0x100+l;
	}
	
    if (!gethexbyte(&sum, c+9, 1))
      return false;
    return true;
  }
  
  if (nType==1){
    if (!gethexbyte(&sum, c+9, 1))
      return false;
    m_bEnd=true;
    return true;
  }
  
  if (nType==4)
    return true;
  
  fprintf(stderr, "Not support type 03-05 of Intel HEX format\n");
  return false;
}

char *intelhex::SetData(int *p, int n, int nAddr, int mask, char *r){
  bool bIgnore=true;;
  
  int sum=n*2;
  sum += (nAddr >> 8) + nAddr & 0xff; 
  sprintf(r,":%02X%04X00",n*2, nAddr);
  for (int i=0;i<n;i++){
    if (p[i]!=mask)
      bIgnore=false;
    
    sprintf(r+9+i*4, "%02X%02X", p[i] & 0xff, (p[i] >> 8) & 0xff) ;
    sum += (p[i]>>8) + (p[i] & 0xff);
  }
  sprintf(r+9+n*4, "%02X", (0x100-sum) & 0xff);
  
  if (bIgnore)
    r[0]=0;
  
  return r;
}

////////////////////////////////////////////////////////////

bool picmemory::Load(FILE *fp){
  char buf[1024];
  
  while (fgets(buf, sizeof(buf), fp)){
    chopstr(buf);
    if (buf[0]){
      if (buf[0]==':'){
		if (!m_pHexLine)
		  m_pHexLine = new intelhex(this);
		if (!m_pHexLine->GetData(buf)){
		  fprintf(stderr, "[ERROR] Wrong file format.\n");
		  delete m_pHexLine;
		  m_pHexLine=NULL;
		  return false;
		}
		
		if (m_pHexLine->m_bEnd)
		  break;
      } else {
		fprintf(stderr, "[ERROR] Wrong/not support file format.\n");
		delete m_pHexLine;
		m_pHexLine=NULL;
		return false;
      }
    }
  }
  if (m_pHexLine)
    delete m_pHexLine;
  m_pHexLine=NULL;
  return true;
}

bool picmemory::Save(FILE *fp){
  char buf[1024];
  if (!m_pHexLine)
    m_pHexLine = new intelhex(this);
  
  if (m_pProgArea){
    int size=m_pDevice->GetProgSize();
    int mask=m_pDevice->GetPMMask();
    for (int i=0;i<size;){
      int n=DUMPMEMSIZE;
      if (i+DUMPMEMSIZE > size)
		n = size-i;
      m_pHexLine->SetData(m_pProgArea+i, n, PROGMEMADDR+i*2, mask, buf);
      if (buf[0])
		fprintf(fp, "%s\n", buf);
      i+=n;
    }
  }
  
  if (m_pConfArea){
    int size=m_pDevice->GetConfSize();
    int mask=m_pDevice->GetConfMask1();
    for (int i=0;i<size;){
      int n=DUMPMEMSIZE;
      if (i+DUMPMEMSIZE > size)
		n = size-i;
      m_pHexLine->SetData(m_pConfArea+i, n, CONFMEMADDR+i*2, mask, buf);
      if (buf[0])
		fprintf(fp, "%s\n", buf);
      i+=n;
    }
  }
  
  if (m_pDataArea){
    int size=m_pDevice->GetDataSize();
    int mask=DATAMASK;
    for (int i=0;i<size;){
      int n=DUMPMEMSIZE;
      if (i+DUMPMEMSIZE > size)
		n = size-i;
      m_pHexLine->SetData(m_pDataArea+i, n, DATAMEMADDR+i*2, mask, buf);
      if (buf[0])
		fprintf(fp, "%s\n", buf);
      i+=n;
    }
  }
  
  fprintf(fp, ":00000001FF\n");
  
  if (m_pHexLine)
    delete m_pHexLine;
  m_pHexLine=NULL;
  return true;
}

bool picmemory::Init(device *pD){
  if (!pD)
    return false;
  
  m_pDevice=pD;
  
  if (m_pDevice->GetProgSize() >= (CONFMEMADDR-PROGMEMADDR)/2)
    return false;
  if (m_pDevice->GetConfSize() >= (DATAMEMADDR-CONFMEMADDR)/2)
    return false;
  if (m_pDevice->GetDataSize() >= 0x1000) // for margin
    return false;
  
  if (m_pDevice->GetProgSize())
    m_pProgArea=new int [m_pDevice->GetProgSize()];
  if (m_pDevice->GetDataSize())
    m_pDataArea=new int [m_pDevice->GetDataSize()];
  if (m_pDevice->GetConfSize())
    m_pConfArea=new int [m_pDevice->GetConfSize()];
  
  for (int i=0;i<m_pDevice->GetProgSize();i++)
    m_pProgArea[i]=m_pDevice->GetPMMask();
  
  for (int i=0;i<m_pDevice->GetDataSize();i++)
    m_pDataArea[i]=DATAMASK;
  
  for (int i=0;i<m_pDevice->GetConfSize();i++)
    m_pConfArea[i]=m_pDevice->GetConfMask1();
  
  m_nProgSize2=m_pDevice->GetProgSize()*2;
  m_nDataSize2=m_pDevice->GetDataSize()*2;
  m_nConfSize2=m_pDevice->GetConfSize()*2;
  
  return true;
}
