// picmemory.h
//    2007.6 Y.Senta

#ifndef _HEXFILE_H_
#define _HEXFILE_H_

#define LINEDATASIZE 128

#define PROGMEMADDR 0x0000
#define CONFMEMADDR 0x4000
#define DATAMEMADDR 0x4200

class intelhex;
class picmemory;
class device;

class hexline{
  friend class intelhex;
  picmemory *m_pMem;
  
  bool GetWord(int *, const char *, int, int);
  bool GetByte(int *, const char *, int, int);
 public:
  hexline(){
    m_pMem=NULL;
  }
  ~hexline(){}
  
  bool m_bEnd;
  
  virtual bool GetData(const char *){
    return false;
  }
  virtual char *SetData(int *, int, int, int, char *p){
    p[0]=0;
    return p;
  }
};

class intelhex:public hexline {
 public:
  intelhex(){}
  intelhex(picmemory *p){
    m_pMem=p;
  }
  ~intelhex(){}
  bool GetData(const char *);
  char *SetData(int *, int, int, int, char *);
};

class picmemory{
  int m_nProgSize2;
  int m_nConfSize2;
  int m_nDataSize2;
  
  int *m_pProgArea;
  int *m_pDataArea;
  int *m_pConfArea;
  
  hexline *m_pHexLine;
  device  *m_pDevice;
 public:
  int m_nCalibData1, m_nCalibData2;
  
  picmemory(){
	m_nProgSize2=0;
	m_nConfSize2=0;
	m_nDataSize2=0;
	
    m_pProgArea=NULL;
    m_pDataArea=NULL;
    m_pConfArea=NULL;
	
    m_pHexLine=NULL;
  }
  ~picmemory(){
    if (m_pProgArea)
      delete [] m_pProgArea;
    if (m_pDataArea)
      delete [] m_pDataArea;
    if (m_pConfArea)
      delete [] m_pConfArea;
    if (m_pHexLine)
      delete m_pHexLine;
  }
  
  device  *GetDevice(){
    return m_pDevice;
  }
  
  bool Init(device *);
  bool Load(FILE *);
  bool Save(FILE *);
  
  int GetProgSize2(){
	return m_nProgSize2;
  }
  int GetConfSize2(){
	return m_nConfSize2;
  }
  int GetDataSize2(){
	return m_nDataSize2;
  }
  int *GetProgMem(){
	return m_pProgArea;
  }
  int *GetConfMem(){
	return m_pConfArea;
  }
  int *GetDataMem(){
	return m_pDataArea;
  }
};
#endif
