// picw509.cc
//    2007.6 Y.Senta

#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include "picw509env.h"
#include "common.h"

#define DEF_DEVICE "/dev/ttyS0"
#define DEF_BPS "38400"

#ifndef CHIPPROPATY
#define CHIPPROPATY "picw509.cfg"
#endif

#define VERSION "0.2"

int g_nDebugLevel=0;

void handler(int){
  fprintf(stderr, " ^C\n");
}

void help(int n){
  if (n==1)
	fprintf(stderr, "[ERROR]: Invalid option(s)\n\n");
  if (n==3)
	fprintf(stderr, "[ERROR]: No additional information\n\n");
  
  fprintf(stderr, "Usage: picw509 <-k/e/p/r/v/q> <-i/o/f PATH> <-d PORT> <-c CHIP>\n");
  
  fprintf(stderr, "  -e: Erase\n");
  fprintf(stderr, "  -p: Program data\n");
  fprintf(stderr, "  -r: Read data\n");
  fprintf(stderr, "  -v: Verify data\n");
  fprintf(stderr, "  -c: Chip select\n");
  fprintf(stderr, "  -d: Device setting (Default: %s)\n", DEF_DEVICE);
  fprintf(stderr, "  -i: Input hex file (\"-\" means STDIN)\n");
  fprintf(stderr, "  -o: Onput hex file (\"-\" means STDOUT)\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "  -f: device propaty File (Default: %s)\n", CHIPPROPATY);
  fprintf(stderr, "  -k: checK hardware connection\n");
  fprintf(stderr, "  -q: Quite mode\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "  -s: communication Speed (Default: 38400)\n");
  fprintf(stderr, "  -x: eXchange hardware bps\n");
  fprintf(stderr, "  -l: Low voltage programming mode\n");
  fprintf(stderr, "  -n: No advance writing mode\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "  -b: deBug lebel\n");
  fprintf(stderr, "  -h: Help");
  if (n==2)
	fprintf (stderr, "... you know this one.");
  fprintf(stderr, "\n");
}

int main(int argc, char **argv){
  picw509env env;
  picw509cmd *pCmd=new picw509cmd;
  int nHelpLevel=0;
  char *pDefFile=NULL;
  
  for (int i=1;i<argc;i++){
    char cCmd=0;
    if (argv[i][0]=='-'){
      int j=1;
      while (argv[i][j] && !nHelpLevel){
		switch(argv[i][j]){
		case 'k':
		  pCmd->m_bCheck=true;
		  break;
		case 'e':
		  pCmd->m_bErase=true;
		  break;
		case 'p':
		  pCmd->m_bErase=true;
		  pCmd->m_bWrite=true;
		  break;
		case 'v':
		  pCmd->m_bVerify=true;
		  break;
		case 'r':
		  pCmd->m_bRead=true;
		  break;
		case 'l':
		  pCmd->m_bLVP=true;
		  break;
		case 'n':
		  pCmd->m_bAdvance=false;
		  break;
		case 'q':
		  pCmd->m_bQuite=true;
		  break;
		case 'd':
		case 'c':
		case 's':
		case 'i':
		case 'o':
		case 'f':
		case 'x':
		case 'b':
		  cCmd=argv[i][j];
		  break;
		case 'h':
		  nHelpLevel=2;
		  break;
		default:
		  nHelpLevel=1;
		  break;
		}
		j++;
		
		if (cCmd){
		  if (argv[i][j])
			nHelpLevel=3;
		  break;
		}
      }
    } else {
      nHelpLevel=1;
    }
	
    if (cCmd){
      i++;
      if (i==argc){
		nHelpLevel=3;
      } else {
		char *p=setstring(argv[i]);
		switch(cCmd){
		case 'd':
		  pCmd->m_pDeviceName=p;
		  break;
		case 'c':
		  pCmd->m_pChipName=p;
		  break;
		case 's':
		  pCmd->m_pBPS=p;
		  break;
		case 'i':
		  pCmd->m_pInputHEXPath=p;
		  break;
		case 'o':
		  pCmd->m_pOutputHEXPath=p;
		  break;
		case 'x':
		  pCmd->m_pNewBPS=p;
		  break;
		case 'f':
		  pDefFile=p;
		  break;
		case 'b':
		  g_nDebugLevel=atoi(p);
		  delete [] p;
		  break;
		}
      }
    }
  }
  
  if (!pCmd->m_bQuite)
	fprintf(stderr, "PIC Writer509 Ver." VERSION "\n\n");
  if (g_nDebugLevel){
    dbprintf(1, "DebugLevel %d\n", g_nDebugLevel);
    pCmd->DBDump();
    if (pDefFile)
      dbprintf(1, "ChipDefFile  [%s]\n",pDefFile);
  }
  
  if (!pDefFile)
    pDefFile=setstring(CHIPPROPATY);
  
  if (!pCmd->m_pDeviceName)
    pCmd->m_pDeviceName=setstring(DEF_DEVICE);
  
  if (!pCmd->m_pBPS)
    pCmd->m_pBPS=setstring(DEF_BPS);

  signal(SIGINT, handler);
  
  if (env.Init(pDefFile)){
    if (nHelpLevel){
      help(nHelpLevel);
      env.DumpDevice();
    } else {
      env.DoCmd(pCmd);
    }
  }
  
  if (pDefFile)
	delete [] pDefFile;
  if (pCmd)
	delete pCmd;
  return 0;
}
