// picw509env.cc
//    2007.6 Y.Senta

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#include "device.h"
#include "picw509env.h"
#include "serialport.h"
#include "piccommand.h"
#include "configstr.h"
#include "common.h"

void picw509cmd::DBDump(){
  dbprintf(3, "Input Commdand:");
  if (m_bCheck)
    dbprintf(3, " Check");
  if (m_bErase)
    dbprintf(3, " Erase");
  if (m_bWrite)
    dbprintf(3, " Write");
  if (m_bVerify)
    dbprintf(3, " Verify");
  if (m_bRead)
    dbprintf(3, " Read");
  
  dbprintf(3, "\n");
  if (m_bLVP)
    dbprintf(3, "Low Voltage mode\n");
  if (m_bQuite)
    dbprintf(3, "sirent mode\n");
  
  if (m_pBPS)
    dbprintf(3, "BPS [%s]\n", m_pBPS);
  if (m_pDeviceName)
    dbprintf(3, "Device [%s]\n", m_pDeviceName);
  if (m_pChipName)
    dbprintf(3, "Chip [%s]\n", m_pChipName);
  if (m_pNewBPS)
    dbprintf(3, "NEW BPS [%s]\n", m_pNewBPS);
  if (m_pInputHEXPath)
    dbprintf(3, "Input File [%s]\n", m_pInputHEXPath);
  if (m_pOutputHEXPath)
    dbprintf(3, "Output File [%s]\n", m_pOutputHEXPath);

  if (m_bAdvance)
    dbprintf(3, "Advance Writing mode\n");
}

bool picw509cmd::ChipAccessCmd(){
  if (m_bErase || m_bWrite || m_bVerify || m_bRead)
    return true;
  return false;
}

bool picw509cmd::isCommand(){
  if (m_bCheck || m_bErase || m_bWrite || m_bVerify || m_bRead || m_pNewBPS)
    return true;
  return false;
}

///////////////////

picw509env::picw509env(){
  m_pDeviceList=NULL;
}

picw509env::~picw509env(){
  if (m_pDeviceList)
  	delete m_pDeviceList;
}

bool picw509env::Init(const char *szPath){
  dbprintf(1, "Load %s\n", szPath);
  if (!LoadDeviceList(szPath)){
	return false;
  }
  return true;
}

bool picw509env::LoadDeviceList(const char *szPath){
  if (m_pDeviceList)
	delete m_pDeviceList;
  m_pDeviceList=NULL;
  
  char buf[1024];
  
  FILE *fp=fopen(szPath, "r");
  
  if (!fp){
	fprintf(stderr, " [ERROR]: Can not open define file\n");
	return false;
  }
  
  int nLen=strlen(DEFSTARTSTR);
  while (fgets(buf, sizeof(buf), fp)){
	if (buf[0]==COMMENTCHAR)
	  continue;
	
	if (!strncmp(buf, DEFSTARTSTR, nLen)){
	  char *p=chopstr(buf+nLen);
	  dbprintf(2, " DeviceLoad [%s]\n", p);
	  
	  device *pTmp=new device(p);
	  pTmp->m_pNext=m_pDeviceList;
	  m_pDeviceList=pTmp;
	  
	  if (!pTmp->Load(fp)){
		fprintf(stderr, " [ERROR]: [%s] Define error\n", p);
		fclose(fp);
		return false;
	  }
	} else {
	  fprintf(stderr, " [ERROR]: Wrong formatted Define file\n");
	  fclose(fp);
	  return false;
	}
  }
  fclose(fp);
  return true;
}

void picw509env::DumpDevice(){
  fprintf(stderr, "Supported Device: ");
  
  device *pTmp=m_pDeviceList;
  while (pTmp){
	fprintf(stderr, "%s ", pTmp->GetName());
	pTmp=pTmp->m_pNext;
  }
  fprintf(stderr, "\n");
}

device *picw509env::SearchChip(const char *c){
  if (!c)
	return NULL;
  
  device *pTmp=m_pDeviceList;
  
  while (pTmp){
	if (!strcmp(pTmp->GetName(), c))
	  return pTmp;
	
	pTmp=pTmp->m_pNext;
  }
  return NULL;
}

bool picw509env::DoCmd(picw509cmd *pCmd){
  if (!pCmd)
	return false;
  
  if (!pCmd->isCommand()){
    if (!pCmd->m_bQuite)
      fprintf(stderr, "nothing to do\n");
    return true;
  }
  
  device *pDev=SearchChip(pCmd->m_pChipName);
  
  if (pCmd->ChipAccessCmd()){
    if (!pCmd->m_pChipName)
      return false;
    if (!pDev){
      fprintf(stderr, "[ERROR]: [%s] is not supported Chip\n", pCmd->m_pChipName);
      return false;
    }
  }
  
  int nNewBpsNo=0;
  if (pCmd->m_pNewBPS){
    if (!strcmp(pCmd->m_pNewBPS, "9600"))
      nNewBpsNo=0;
    else if (!strcmp(pCmd->m_pNewBPS, "19200"))
      nNewBpsNo=1;
    else if (!strcmp(pCmd->m_pNewBPS, "38400"))
      nNewBpsNo=2;
    else if (!strcmp(pCmd->m_pNewBPS, "57600"))
      nNewBpsNo=3;
    else if (!strcmp(pCmd->m_pNewBPS, "115200"))
      nNewBpsNo=4;
    else {
      fprintf(stderr, "[ERROR]: Can not change [%s]bps\n", pCmd->m_pBPS);
      return false;
    }
  }
  
  if (pCmd->m_bRead && !pCmd->m_pOutputHEXPath){
    fprintf(stderr, "[ERROR]: Read command requires file path\n");
    return false;
  }
  
  if ((pCmd->m_bWrite || pCmd->m_bVerify) && !pCmd->m_pInputHEXPath){
    fprintf(stderr, "[ERROR]: Program/Verify command requires file path\n");
    return false;
  }
  
  FILE *fpi=NULL;
  FILE *fpo=NULL;
  
  if (pCmd->m_pInputHEXPath){
    if (strcmp(pCmd->m_pInputHEXPath, "-")){
      fpi=fopen(pCmd->m_pInputHEXPath, "r");
      if (!fpi){
		fprintf(stderr, "[ERROR]: Can not open [%s]\n", pCmd->m_pInputHEXPath);
		return false;
      }
    } else {
      fpi=stdin;
    }
  }
  
  if (pCmd->m_pOutputHEXPath){
    if (strcmp(pCmd->m_pOutputHEXPath, "-")){
      fpo=fopen(pCmd->m_pOutputHEXPath, "w");
      if (!fpo){
		fprintf(stderr, "[ERROR]: Can not open [%s]\n", pCmd->m_pOutputHEXPath);
		if (pCmd->m_pInputHEXPath)
		  fclose(fpi);
		return false;
      }
    } else {
      fpo=stdout;
    }
  }
  
  serialport *pPort=new serialport;
  if (pPort->OpenSerialPort(pCmd->m_pDeviceName, pCmd->m_pBPS)){
    piccommand *pPic = new piccommand;
	
    pPic->m_bCheck=pCmd->m_bCheck; 
    pPic->m_bErase=pCmd->m_bErase; 
    pPic->m_bWrite=pCmd->m_bWrite;
    pPic->m_bVerify=pCmd->m_bVerify;
    pPic->m_bRead=pCmd->m_bRead;
    pPic->m_bLVP=pCmd->m_bLVP;
    pPic->m_bQuite=pCmd->m_bQuite;
	
    pPic->m_bAdvance=pCmd->m_bAdvance;

    pPic->m_pDevice=pDev;
    pPic->m_pSerial=pPort;
    pPic->m_pFHexI=fpi;
    pPic->m_pFHexO=fpo;
	
    if (pCmd->m_pNewBPS)
      pPic->m_bChgBps=true;
	else
      pPic->m_bChgBps=false;
    pPic->m_nNewBps=nNewBpsNo;
	
    pPic->Command();
	
    delete pPic;
    pPort->CloseSerialPort();
  }
  
  delete pPort;
  
  if (pCmd->m_pInputHEXPath)
    fclose(fpi);
  if (pCmd->m_pOutputHEXPath){
    fclose(fpo);
	if (strcmp(pCmd->m_pOutputHEXPath, "-"))
	  chown(pCmd->m_pOutputHEXPath, getuid(), getgid());
  }
  
  return true;
}
