// serialport.cc
//    2007.6 Y.Senta

#include <stdio.h>
#include <string.h>
#include<termios.h>
#include<fcntl.h>
#include<unistd.h>

#include "serialport.h"

#define TIMEOUT 200000

serialport::serialport(){
  m_nFD=0;
  m_pBackupTM=NULL;
}

serialport::~serialport(){
  CloseSerialPort();
}

bool serialport::OpenSerialPort(const char *szDev, const char *szBps){
  int nBps;
  
  if (!strcmp(szBps, "9600"))
	nBps=B9600;
  else if (!strcmp(szBps, "19200"))
	nBps=B19200;
  else if (!strcmp(szBps, "38400"))
	nBps=B38400;
  else if (!strcmp(szBps, "57600"))
	nBps=B57600;
  else if (!strcmp(szBps, "115200"))
	nBps=B115200;
  else {
	fprintf(stderr, "[ERROR]: not support [%s]bps\n");
	return false;
  }
  
  CloseSerialPort();
  
  if((m_nFD = open(szDev, O_RDWR | O_NOCTTY ))==-1){
	perror(szDev);
	return false;
  }
  m_pBackupTM=new termios;
  tcgetattr(m_nFD, m_pBackupTM);
  
  termios NewTM;
  bzero(&NewTM, sizeof(NewTM));
  NewTM.c_cflag = (nBps | CS8 | CLOCAL | CREAD);
  NewTM.c_oflag = 0;
  
  tcflush(m_nFD, TCIFLUSH);
  tcsetattr(m_nFD,TCSANOW,&NewTM);
  
  return true;
}

void serialport::CloseSerialPort(){
  if (m_nFD){
	if (m_pBackupTM)
	  tcsetattr(m_nFD, TCSANOW, m_pBackupTM);
	close(m_nFD);
	m_nFD=0;
  }
  if (m_pBackupTM){
	delete m_pBackupTM;
	m_pBackupTM=NULL;
  }
}

bool serialport::SendChar(char c){
  if (m_nFD)
	return write(m_nFD, &c, 1)==1 ? true : false;
  return false;
}

bool serialport::SendCmd(char c){
  char tmp;
  if (!SendChar(c))
	return false;
  if (!RecvChar(&tmp))
	return false;
  if (c!=tmp)
	return false;
  
  return true;
}

bool serialport::RecvChar(char *c){
  if (m_nFD){
	fd_set rfds;
	struct timeval tfds;
	  
	FD_ZERO(&rfds);
	FD_SET(m_nFD, &rfds);
	
	tfds.tv_sec  = 0;
	tfds.tv_usec = TIMEOUT;
	
	if (!select(m_nFD+1, &rfds, NULL, NULL, &tfds))
	  return false;
	
	if (FD_ISSET(m_nFD, &rfds))
	  return read(m_nFD, c, 1) == 1 ? true : false;
  }
  return false;
}

int serialport::RecvChar(char *c, int m){
  int ret=0;
  
  if (m_nFD){
	fd_set rfds;
	struct timeval tfds;
	
	tfds.tv_sec  = 0;
	tfds.tv_usec = TIMEOUT;
	
	for (ret=0; ret<m; ret++){
	  FD_ZERO(&rfds);
	  FD_SET(m_nFD, &rfds);
	  
	  if (!select(m_nFD+1, &rfds, NULL, NULL, &tfds))
		return ret;
	  
	  if (FD_ISSET(m_nFD, &rfds)){
		if (read(m_nFD, c+ret, 1)!=1)
		  return 0;
	  } else {
		return 0;
	  }
	}
  }
  return ret;
}
